/* Program using the PAR card and Imagemaster to fade titles over video. 

By Don Cox, Dec 94. Additions, May 95, June 95. Bug fix, Dec 95. Copyright, not Public Domain.
Best results will be obtained if titles match the size of the PAR images (752 by 576 in PAL) and do not have to be resized. 
Optionally, titles can be colour keyed, and the video image can be rolled so that the relevant area appears in an aperture. */
/* $VER: PARtitleFader Dec 95 */

/*call open("STDERR","ram:tracePF","W")
trace r*/


if show('P','IM_Port') = 0 then do
    say "Can't find image processor's ARexx port!!!"; /* not running? */
    say "This script requires ImageMaster to run!";
    exit(20);
    end;

if ~show('L',"rexxsupport.library") then do
  if ~addlib('rexxsupport.library',0,-30,0) then do
    say 'We Have A Library Problem, Unable To Load "rexxsupport.library"';
    say 'Cannot operate this script without this library - sorry!';
    'finish';
    exit 10;
    end;
  end;


options results
call pragma('stack',20000);

address 'IM_Port'
'imtofront'; /* show user the IM screen */
'filerequest "DDR:" "" "" "Background Video file"'
PARanimfile = result
if PARanimfile = 'FR_CANCELLED' then do
    address 'IM_Port'
    'imtofront'
    exit 0
    end
PARanimfile = expandfilename(PARanimfile)
PARanimpath = gimmepath(PARanimfile)
parse var PARanimpath PARanimpath PARanimname
PARanimname = strip(PARanimname)

'filerequest "DDR:" "" "" "Final Video file"'
PARoutputfile = result
if PARoutputfile = 'FR_CANCELLED' then wayout("No output selected")
PARoutputfile = expandfilename(PARoutputfile)
PARoutputpath = gimmepath(PARoutputfile)
parse var PARoutputpath PARoutputpath PARoutputname
PARoutputname = strip(PARoutputname)

'filerequest "Art:" "TestTitle" "" "First title file"'
titlefile = result
if titlefile = 'FR_CANCELLED' then wayout("No title file selected")
titlefile = expandfilename(titlefile)
titlepath = gimmepath(titlefile)
parse var titlepath titlepath titlename
titlename = strip(titlename)

'asknumber "How many titles?" 1'
ntitles = result


titlenumber = "" /* defaults */
titlebase = titlename
trailer = ""

if ntitles>1 then do
/* strip off number from end of filename */
    do i = 1 to length(titlename) /* First look for trailers such as .LBM */
        trailer = right(titlename,i)
        if verify(trailer,"0123456789", "m") ~=0 then break
        end
    trailer = right(trailer,length(trailer)-1)
    filenameshort = left(titlename, length(titlename)-length(trailer))

    do i = 1 to length(filenameshort) /* now find the frame number */
        endofname = right(filenameshort,i)
        if verify(endofname,"0123456789") ~=0 then break
        end
    numberlength = length(endofname)-1


    titlenumber = substr(endofname,2)
    titlebase = left(filenameshort,length(filenameshort)-numberlength)
    end

address DDR

file PARanimfile
if result = "" then wayout("Could not load video file")
frames
nframes = result
if nframes = 0 then wayout()


/* get PAR prefs (and restore them at exit) */

Mode
r_Mode=result
EMode
r_EMode=result
Filter
r_filter=result
Translate
r_Translate=result
Exporter
r_Exporter=result

/* Set up prefs */
Exporter IFF
EMode FRAME
Mode ANIM
Filter OFF
Translate ON

address 'IM_Port'
'asknumber "First Frame of Video to be affected?" 1'
firstframe = result
'asknumber "Last Frame?" '||nframes
lastframe = result
if lastframe>nframes then wayout()
if lastframe<firstframe then wayout()
CompFrames = lastframe-firstframe+1

/* We can fade the video in and out in an area of a still image, or the usual fade of still titles over video */
videofade = 0
if ntitles = 1 then do
    'askboolean "Fade title" "Fade video" 0' /* quotes??? */
    videofade = result
    end

FramesPerTitle = trunc((CompFrames)/ntitles)
'trenddraw "FadePercent" 1 '||FramesPerTitle||' 0 100 100 "Fade Envelope"'


'gadgets "Roll Video", "in X", "Roll Video", "in Y", "Roll Video", "in X and Y", "No Roll", "" '
rolltype = result
if (rolltype = 1|rolltype = 3) then 'trenddraw "RollX" 1 '||CompFrames||' -752 752 0 "Video Roll Envelope X"'
if (rolltype = 2|rolltype = 3) then 'trenddraw "RollY" 1 '||CompFrames||' -576 576 0 "Video Roll Envelope Y"'


PARanimframe = right(firstframe, 4, 0) /* pad with zeros */
'imagepath 'titlepath
'imageext ""'

titlename = titlebase||titlenumber||trailer
'loadasprimary '||'"'titlename'"'
success = rc

'gadgets "Key Color", "Top Left", "Sample for","Key Color", "No Key", "Color"'
KeyType = result
'rect 1 1 3 3' /*sample top left corner of title */
if KeyType = 2 then 'rectarea'
'cdef'
parse var result red','green','blue','average','variance


'entire'
/*'autoredraw 0'*/ /* OFF */
'autoactivate 0'

do i = 1 to ntitles
    address 'IM_Port'
    'secondary'
    buffdata = result
    parse var buffdata bname ',' bnum ',' bx ',' by ',' btot ',' bmem ',' bparname ',' bparnum
    if bname ~= '<none>' then 'killbuff '||bnum /* clear ram for new image */
    'imagepath 'titlepath
    titlename = titlebase||titlenumber||trailer
    'loadassecondary '||'"'titlename'"'
    success = rc
    if ntitles>1 then titlenumber = right(titlenumber+1, numberlength, 0)

    do f= 1 to FramesPerTitle
        address 'IM_Port'
        call buffkill()
        'current'
        buffdata = result
        parse var buffdata bname ', ' bnum ', ' bx ', ' by ', ' btot ', ' bmem ', ' bparname ', ' bparnum
        if bname ~= '<none>' then 'killbuff' bnum /* clear ram for next video image */

        'imagepath 'PARanimpath
        'imageext ""'
        'loadasprimary '||'"'||PARanimname||"."||PARanimframe||'"'

/* Rolling the image allows a selected part of it to be placed in an aperture in the image in front */
        Xroll = 0
        Yroll = 0
        if (rolltype = 1|rolltype = 3) then do
            'trendget "RollX" 'f /* f is frame number */
            Xroll = result
            end
        if (rolltype = 2|rolltype = 3) then do
            'trendget "RollY" 'f
            Yroll = result
            end
        if rolltype~=4 then 'rollimage 'Xroll Yroll 1

        'trendget "FadePercent" '||f  /* f is frame number for this title */
        pc = result*2.55
        call blendset(pc)
        'entire'
        if videofade = 1 then do /* if video is to be faded, swap primary and secondary */
            'current'
            buffdata = result
            parse var buffdata videoname ', ' videonum ', ' videoX ', ' videoY ', ' stuff
            'secondary'
            buffdata = result
            parse var buffdata titlename ', ' titlenum ', ' titleX ', ' titleY ', ' stuff
            'newcurrent 'titlenum /* swap primary and secondary */
            'newsecondary 'videonum
            if (titleX~=videoX|titleY~=videoY) then do /* title not video size */
                call buffkill()
                'newbuf 'videoX videoY "scaled"
                scalenum = result /* number of buffer */
                'newcurrent 'scalenum
                'newsecondary 'titlenum
                'entire'
                call blendset(255)
                'merge' /* scales title to video size */
                'killbuff 'titlenum
                'newsecondary 'videonum
                call blendset(pc)
                end
            if KeyType = 3 then 'merge' /* No keying - just fade */
            else 'colkeymin'
            'current'
            buffdata = result
            parse var buffdata priname ', ' prinum ', ' stuff
            'save 'prinum '"'||PARoutputfile||'"'
            'undo'
            'newsecondary 'prinum
            'killbuff 'videonum /* clear out the video frame */
            call buffkill()
            end

        if videofade ~=1 then do
            if KeyType = 3 then 'merge' /* No keying - just fade */
            else 'colkeysmax'
            'current'
            buffdata = result
            parse var buffdata bname ', ' bnum ', ' stuff
            'save 'bnum '"'||PARoutputfile||'"'
            end


        PARanimframe = right(PARanimframe+1, 4, 0) /* set number for next frame */

        end
    end 
wayout("Done - now name new PAR anim")


wayout:
address 'IM_Port'
'autoredraw 1'  /* ON */
'autoactivate 1'

parse arg infomessage
if infomessage ~= "" then do
    address 'IM_Port'
    'message 'infomessage
    end
address DDR
TOFRONT
Mode r_Mode
EMode r_EMode
Filter r_filter
Translate r_Translate
Exporter r_Exporter
Done
exit 0
end

/* +++++++++++++++++++++++++++++++  +++++++++++++++++++++++++++++++++++++ */

/* gimmepath
 This takes the provided argument and sucks the path out of it, then returns that path to the caller, sans file name. */

gimmepath:
  parse arg fullnamegx;
    tempgx = reverse(fullnamegx);
    lengx = length(fullnamegx);   /* get length of string */
    slashdex = index(tempgx,'/'); /* first occurance of '/' from right */
    colondex = index(tempgx,':');  /* first occurance of ':' from right */
    seploc = 0; /* assumes current dir, no path supplied */
    if slashdex ~= 0 then do /* we assume we are in a DIR */
      seploc = (lengx - slashdex)+1;
      end;
    else do
      if colondex ~= 0 then do /* we assume we are on a device */
        seploc = (lengx - colondex)+1;
        end;
      end;
  gxname = substr(fullnamegx,seploc+1); /* if you ever need it */
  gxpath = left(fullnamegx,seploc)
  gxpath = gxpath" "gxname
  return(gxpath);


/* +++++++++++++++++++++++++++++++ ++++++++++++++++++++++++++++++++++++ */

/*
 * Since this script can't be expected to know where the CD of the user
 * is when this cmd is invoked, we have to check the path the user
 * provides - if it's not specified right from a root, then we have
 * to make it a complete specification from the root.
 */
expandfilename:
  parse arg jfile;
  if index(jfile,':') = 0 then do
    curdir = pragma(D);
    if right(curdir,1) ~= ':' then do
      if right(curdir,1) ~= '/' then do
        if curdir ~= '' then do
          curdir = curdir || '/';
          end;
        end;
      end;
    jfile = curdir||jfile;
    end;
  return(jfile);

/* +++++++++++++++++++++++++++++++++ ++++++++++++++++++++++++++++++++++ */

blendset:
parse arg bv
'setallblend 'bv
'setblend 1 255 'bv /* bug workaround - setallblend doesn't */
'setblend 2 255 'bv
'setblend 3 255 'bv
'setblend 4 255 'bv
return

/* +++++++++++++++++++++++++++++ +++++++++++++++++++++++++++++++++++++ */

buffkill: procedure
'current'
buffdata = result
parse var buffdata currentname ', ' currentnum ', ' stuff
'sbuffers'
buffdata = result
do until buffdata =""
    parse var buffdata bname ', 'bnum ', ' buffdata
    if bname ~= '<none>' then do
        'newcurrent 'bnum
        'killbuff' bnum /* general cleanout */
        end
    end
'newcurrent 'currentnum
'sbuffers' /* check */
buffdata = result
return

